from . import _SQ1_SCRAMBLER, _UTIL_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_WCA_scramble():
    """ Gets a WCA scramble for a Square-1. """
    return _SQ1_SCRAMBLER.call("sql_scrambler.getRandomScramble")

@trim
def get_face_turn_metric_scramble(n=40):
    """ Gets a face turn metric scramble of length `n` for a Square-1. Defaults to csTimer's default length of 40. """
    return _UTIL_SCRAMBLER.call("util_scramble.getSquareOneTurnMetricScramble", n)

@trim
def get_twist_metric_scramble(n=20):
    """ Gets a twist metric scramble for a Square-1.  Defaults to csTimer's default length of 20."""
    return _UTIL_SCRAMBLER.call("util_scramble.getSquareOneTwistMetricScramble", n)


from . import _PYRA_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_WCA_scramble():
    """ Gets a WCA scramble of a Pyraminx. """
    return _PYRA_SCRAMBLER.call("pyra_scrambler.getPyraWCAScramble")

@trim
def get_optimal_scramble():
    """ Gets an optimal scramble of a Pyraminx. """
    return _PYRA_SCRAMBLER.call("pyra_scrambler.getPyraOptimalScramble")


from . import _UTIL_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_WCA_scramble(n=70):
    """ Gets a WCA scramble of length `n` for a Megaminx. Defaults to csTimer's default length of 70. """
    return _UTIL_SCRAMBLER.call("util_scramble.getMegaminxWCAScramble", n).replace('\n','').replace('  ',' ').replace('  ',' ')

@trim
def get_Carrot_scramble(n=70):
    """ Gets a Carrot-notation scramble of length `n` for a Megaminx. Defaults to csTimer's default length of 70. """
    return _UTIL_SCRAMBLER.call("util_scramble.getMegaminxCarrotScramble", n).replace('\n','').replace('  ',' ').replace('  ',' ')

@trim
def get_old_style_scramble(n=70):
    """ Gets an old-style scramble of length `n` for a Megaminx. Defaults to csTimer's default length of 70. """
    return _UTIL_SCRAMBLER.call("util_scramble.getMegaminxOldStyleScramble", n)


from . import _MEGA_SCRAMBLER, _223_SCRAMBLER, _133_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_1x1x2_scramble():
    """ Gets a random scramble for a 1x1x2 cuboid (lul). """
    return _MEGA_SCRAMBLER.call("megaScrambler.get112scramble")

@trim
def get_1x3x3_scramble():
    """ Gets a random scramble for a 1x3x3 cuboid. """
    return _133_SCRAMBLER.call("scrambler133.get133scramble")

@trim
def get_floppy_cube_scramble():
    """ Gets a random scramble for a 1x3x3 cuboid. Alias of 1x3x3. """
    return get_1x3x3_scramble()

@trim
def get_super_floppy_cube_scramble():
    """ Gets a random scramble for a 1x3x3 super floppy cuboid. """
    return _MEGA_SCRAMBLER.call("megaScrambler.getSuperFloppyScramble")

@trim
def get_2x2x3_scramble():
    """ Gets a random scramble for a 2x2x3 cuboid. """
    return _223_SCRAMBLER.call("scrambler223.get223scramble")

@trim
def get_3x3x2_scramble():
    """ Gets a random scramble for a 3x3x2 cuboid. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get332scramble")

@trim
def get_3x3x4_scramble():
    """ Gets a random scramble for a 3x3x4 cuboid. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get334scramble")

@trim
def get_3x3x5_scramble(n=25):
    """ Gets a random scramble for a 3x3x5 cuboid, where `n` is the length of the non-3x3x3 portion of the scramble. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get335scramble", n)

@trim
def get_3x3x6_scramble():
    """ Gets a random scramble for a 3x3x6 cuboid. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get336scramble")

@trim
def get_3x3x7_scramble(n=40):
    """ Gets a random scramble for a 3x3x7 cuboid, where `n` is the length of the non-3x3x3 portion of the scramble. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get337scramble", n)


from . import _MEGA_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_WCA_scramble(n=100):
    """ Gets a WCA scramble of length `n` for a 7x7x7 cube. Defaults to csTimer's default length of 100. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get777WCAScramble", n)

@trim
def get_SiGN_scramble(n=100):
    """ Gets a SiGN-notation scramble of length `n` for a 7x7x7 cube. Defaults to csTimer's default length of 100. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get777SiGNScramble", n)

@trim
def get_edges_scramble(n=8):
    """ Gets an edges scramble of length `n` for a 7x7x7 cube. Defaults to csTimer's default length of 8. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get777edgesScramble", n)


from . import _FTO_SCRAMBLER, trim

from random import choice

#------------------------------------------------------------------------------

@trim
def get_random_state_scramble():
    """ Gets a random-state scramble for FTO (face-turning octahedron). """
    return _FTO_SCRAMBLER.call("generate_random_state_scramble")


def get_multiple_random_state_scrambles(n):
    """ Gets `n` random-state scrambles for FTO (face-turning octahedron). """
    scrambles = _FTO_SCRAMBLER.call("generate_multiple_random_state_scrambles", n)
    return [s.strip() for s in scrambles]


@trim
def get_random_moves_scramble(scramble_length = 30):
    """ Returns a random-moves scramble of length `scramble_length` for FTO. """

    def __except(exceptions):
        """ Returns a list range 0-7 except the specified exceptions. """
        if not exceptions:
            return range(8)
        values = list(range(8))
        for n in exceptions:
            values.remove(n)
        return values

    moves = ['F', 'U', 'R', 'L', 'D', 'B', 'BL', 'BR']
    opposites = {
        'U':  'D',
        'D':  'U',
        'F':  'B',
        'B':  'F',
        'L':  'BR',
        'BR': 'L',
        'BL': 'R',
        'R':  'BL'
    }

    scramble = list()
    move_ix = -1

    for _ in range(scramble_length):
        if move_ix == -1:
            # If this is the first move, there are no restrictions on which face we can choose.
            exceptions = None
        else:
            # Otherwise, we'll always omit the face we just chose, so we don't choose it again.
            exceptions = [move_ix]

            if len(scramble) > 1:
                # If the last two faces in the scramble were opposites (ex, U and D), the next move
                # needs to exclude both of those faces so we don't end up with cancellations.
                # ex: U D U' is no good
                # ex: R' BL' R' BL is no good
                # By omitting the two opposite faces from the choices for the next move, the face
                # chosen will intersect those faces, preventing a cancellation.
                face_one_ago = scramble[-1].replace("'", "")
                face_two_ago = scramble[-2].replace("'", "")
                if opposites[face_one_ago] == face_two_ago:
                    exceptions.append(moves.index(face_two_ago))

        # Randomly choose a move from the faces available, ensuring we don't have a cancellation
        move_ix = choice(__except(exceptions))

        # Randomly decide if that turn is clockwise or counterclockwise and apply prime if needed.
        scramble.append(moves[move_ix] + choice(["", "'"]))

    return ' '.join(scramble)



from . import _MEGA_SCRAMBLER, _SKEWB_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_WCA_scramble():
    """ Gets a WCA scramble of a Skewb. """
    return _SKEWB_SCRAMBLER.call("skewb_scrambler.getSkewbWCAScramble")

@trim
def get_ULRB_scramble():
    """ Gets a ULRB scramble of a Skewb. """
    return _MEGA_SCRAMBLER.call("megaScrambler.getSkewbULRBScramble")


from . import _MEGA_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_WCA_scramble(n=80):
    """ Gets a WCA scramble of length `n` for a 6x6x6 cube. Defaults to csTimer's default length of 80. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get666WCAScramble", n)

@trim
def get_SiGN_scramble(n=80):
    """ Gets a SiGN-notation scramble of length `n` for a 6x6x6 cube. Defaults to csTimer's default length of 80. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get666SiGNScramble", n)

@trim
def get_edges_scramble(n=8):
    """ Gets an edges scramble of length `n` for a 6x6x6 cube. Defaults to csTimer's default length of 8. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get666edgesScramble", n)


from . import _222_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_WCA_scramble():
    """ Gets a WCA random scramble (sub-optimal) of a 2x2x2 cube. """
    return _222_SCRAMBLER.call("scramble_222.getRandomScramble")

@trim
def get_random_scramble():
    """ Gets a WCA random scramble (sub-optimal) of a 2x2x2 cube. """
    return _222_SCRAMBLER.call("scramble_222.getRandomScramble")

@trim
def get_optimal_scramble():
    """ Gets an optimal random state scramble of a 2x2x2 cube. """
    return _222_SCRAMBLER.call("scramble_222.getOptimalScramble")


from . import _444_SCRAMBLER, _MEGA_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_random_state_scramble(n=40):
    """ Gets a WCA scramble of length N for a 4x4x4 cube. Defaults to csTimer's default length of 40. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get444WCAScramble", n)

@trim
def get_4BLD_scramble(n=40):
    """ Gets a BLD scramble of length N for a 4x4x4 cube. Alias of get_WCA_scramble. """
    return get_random_state_scramble(n)

@trim
def get_SiGN_scramble(n=40):
    """ Gets a SiGN-notation scramble of length N for a 4x4x4 cube. Defaults to csTimer's default length of 40. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get444SiGNScramble", n)

@trim
def get_WCA_scramble():
    """ Gets a random state scramble of a 4x4x4 cube. """
    return _444_SCRAMBLER.call("scramble_444.getRandomScramble")

@trim
def get_edges_scramble(n=8):
    """ Gets an edges scramble of length `n` for a 4x4x4 cube. Defaults to csTimer's default length of 8. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get444edgesScramble", n)



import execjs
from os import path

#------------------------------------------------------------------------------

def trim(func):
    def trimmed_func(*args, **kwargs):
        return func(*args, **kwargs).strip()
    return trimmed_func

#------------------------------------------------------------------------------

curr_dir = path.dirname(path.realpath(__file__))

with open(path.join(curr_dir, 'js_resources/mathlib.js')) as f:
    MATHLIB_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/cross.js')) as f:
    CROSS_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/scramble.js')) as f:
    SCRAMBLE_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/megascramble.js')) as f:
    MEGA_SCRAMBLE_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/utilscramble.js')) as f:
    UTIL_SCRAMBLE_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/scramble_333_edit.js')) as f:
    SCRAMBLE_333_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/scramble_444.js')) as f:
    SCRAMBLE_444_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/scramble_sq1.js')) as f:
    SCRAMBLE_SQ1_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/2x2x2.js')) as f:
    SCRAMBLE_222_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/2x2x3.js')) as f:
    SCRAMBLE_223_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/1x3x3.js')) as f:
    SCRAMBLE_133_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/pyraminx.js')) as f:
    PYRAMINX_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/skewb.js')) as f:
    SKEWB_SRC = f.read()

with open(path.join(curr_dir, 'js_resources/fto.js')) as f:
    FTO_SRC = f.read()

#------------------------------------------------------------------------------

_UTIL_SCRAMBLER  = execjs.compile(MATHLIB_SRC + SCRAMBLE_SRC + UTIL_SCRAMBLE_SRC)
_PYRA_SCRAMBLER  = execjs.compile(MATHLIB_SRC + PYRAMINX_SRC)
_SQ1_SCRAMBLER   = execjs.compile(MATHLIB_SRC + SCRAMBLE_SRC + SCRAMBLE_SQ1_SRC)
_133_SCRAMBLER   = execjs.compile(MATHLIB_SRC + SCRAMBLE_133_SRC)
_222_SCRAMBLER   = execjs.compile(MATHLIB_SRC + SCRAMBLE_222_SRC)
_223_SCRAMBLER   = execjs.compile(MATHLIB_SRC + SCRAMBLE_223_SRC)
_333_SCRAMBLER   = execjs.compile(MATHLIB_SRC + SCRAMBLE_SRC + CROSS_SRC + SCRAMBLE_333_SRC)
_444_SCRAMBLER   = execjs.compile(MATHLIB_SRC + SCRAMBLE_SRC + SCRAMBLE_333_SRC + SCRAMBLE_444_SRC)
_MEGA_SCRAMBLER  = execjs.compile(MATHLIB_SRC + SCRAMBLE_SRC + SCRAMBLE_333_SRC + MEGA_SCRAMBLE_SRC)
_SKEWB_SCRAMBLER = execjs.compile(MATHLIB_SRC + SCRAMBLE_SRC + SKEWB_SRC)
_FTO_SCRAMBLER   = execjs.compile(FTO_SRC)



from . import _MEGA_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_WCA_scramble(n=60):
    """ Gets a WCA scramble of length `n` for a 5x5x5 cube. Defaults to csTimer's default length of 60. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get555WCAScramble", n)

@trim
def get_5BLD_scramble(n=60):
    """ Gets a BLD scramble of length N for a 5x5x5 cube. Alias of get_WCA_scramble. """
    return get_WCA_scramble(n)

@trim
def get_SiGN_scramble(n=60):
    """ Gets a SiGN-notation scramble of length `n` for a 5x5x5 cube. Defaults to csTimer's default length of 60. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get555SiGNScramble", n)

@trim
def get_edges_scramble(n=8):
    """ Gets an edges scramble of length `n` for a 5x5x5 cube. Defaults to csTimer's default length of 8. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get555edgesScramble", n)


from . import _UTIL_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_WCA_scramble():
    """ Gets a WCA scramble for a Rubik's Clock. """
    return _UTIL_SCRAMBLER.call("util_scramble.getClockWCAScramble")

@trim
def get_Jaap_scramble():
    """ Gets a Jaap-notation scramble for a Rubik's Clock. """
    return _UTIL_SCRAMBLER.call("util_scramble.getClockJaapScramble")

@trim
def get_concise_scramble():
    """ Gets a concise-notation scramble for a Rubik's Clock. """
    return _UTIL_SCRAMBLER.call("util_scramble.getClockConciseScramble")

@trim
def get_efficient_pin_order_scramble():
    """ Gets an efficient pin order scramble for a Rubik's Clock. """
    return _UTIL_SCRAMBLER.call("util_scramble.getClockEfficientPinOrderScramble")


import setuptools

setuptools.setup(
    name="pyTwistyScrambler",
    version="1.3",
    author="euphwes",
    author_email="euphwes@gmail.com",
    url="https://github.com/euphwes/pyTwistyScrambler",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        'appdirs',
        'packaging',
        'PyExecJS',
        'pyparsing',
        'six',
    ],
)



from . import _MEGA_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_8x8x8_scramble(n=120):
    """ Gets a random scramble (SiGN notation) of length `n` for an 8x8x8 cube. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get888scramble", n)

@trim
def get_9x9x9_scramble(n=120):
    """ Gets a random scramble (SiGN notation) of length `n` for a 9x9x9 cube. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get999scramble", n)

@trim
def get_10x10x10_scramble(n=120):
    """ Gets a random scramble (SiGN notation) of length `n` for a 10x10x10 cube. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get101010scramble", n)

@trim
def get_11x11x11_scramble(n=120):
    """ Gets a random scramble (SiGN notation) of length `n` for an 11x11x11 cube. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get111111scramble", n)


from . import _333_SCRAMBLER, _MEGA_SCRAMBLER, trim

#------------------------------------------------------------------------------

@trim
def get_WCA_scramble():
    """ Gets a WCA scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getRandomScramble")

@trim
def get_3BLD_scramble():
    """ Gets a BLD scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.get3BLDScramble")

@trim
def get_random_scramble():
    """ Gets a random scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getRandomScramble")

@trim
def get_edges_scramble():
    """ Gets an edges-only scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getEdgeScramble")

@trim
def get_corners_scramble():
    """ Gets a corners-only scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getCornerScramble")

@trim
def get_LL_scramble():
    """ Gets a last layer scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getLLScramble")

@trim
def get_LSLL_scramble():
    """ Gets a last slot last layer scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getLSLLScramble")

@trim
def get_ZBLL_scramble():
    """ Gets a Zborowski-Bruchem last layer scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getZBLLScramble")

@trim
def get_ZZLL_scramble():
    """ Gets a Zbigniew Zborowski last layer scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getZZLLScramble")

@trim
def get_ZBLS_scramble():
    """ Gets a ZBLS scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getZBLSScramble")

@trim
def get_F2L_scramble():
    """ Gets an F2L (first two layers) scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getF2LScramble")

@trim
def get_LSE_scramble():
    """ Gets an LSE (last six edges) scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getLSEScramble")

@trim 
def get_EOLine_scramble():
    """ Gets an EO line scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getEOLineScramble")

@trim
def get_CMLL_scramble():
    """ Gets a CMLL scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getCMLLScramble")

@trim
def get_CLL_scramble():
    """ Gets a CLL scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getCLLScramble")

@trim
def get_ELL_scramble():
    """ Gets an ELL scramble of a 3x3x3 cube. """
    return _333_SCRAMBLER.call("scramble_333.getELLScramble")

@trim
def get_easy_cross_scramble(n):
    """ Gets an 'easy cross' scramble, where the cross can be solved in `n` moves."""
    return _333_SCRAMBLER.call("scramble_333.getEasyCrossScramble", n)

@trim
def get_2genRU_scramble():
    """ Gets a 2-gen scramble with only RU moves for a 3x3x3 cube. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get333_2genRU_scramble")

@trim
def get_2genLU_scramble():
    """ Gets a 2-gen scramble with only LU moves for a 3x3x3 cube. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get333_2genLU_scramble")

@trim
def get_2genMU_scramble():
    """ Gets a 2-gen scramble with only MU moves for a 3x3x3 cube. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get333_2genMU_scramble")

@trim
def get_3genFRU_scramble():
    """ Gets a 3-gen scramble with only FRU moves for a 3x3x3 cube. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get333_3genFRU_scramble")

@trim
def get_3genRUL_scramble():
    """ Gets a 3-gen scramble with only RUL moves for a 3x3x3 cube. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get333_3genRUL_scramble")

@trim
def get_3genRrU_scramble():
    """ Gets a 3-gen scramble with only RrU moves for a 3x3x3 cube. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get333_3genRrU_scramble")

@trim
def get_half_turns_scramble():
    """ Gets a half turns-only scramble for a 3x3x3 cube. """
    return _MEGA_SCRAMBLER.call("megaScrambler.get333_halfTurns_scramble")